﻿// NotesSearchPlugin.cs Version 1.6 (04/27/2024)
// Author: Gerd Besch | www.besch-it.com | code@besch-it.com
//         Hauptstrasse 27 | 72532 Gomadingen | Germany
//
// Developed with SharpDevelop V4.4.1 Build 9729
//
// SharpDevelop References:
// Keepass --> current KeePass.exe
// System --> C:\WINDOWS\Microsoft.Net\assembly\GAC_MSIL\System\v4.0_4.0.0.0__b77a5c561934e089\System.dll
// System.Drawing --> C:\WINDOWS\Microsoft.Net\assembly\GAC_MSIL\System.Drawing\v4.0_4.0.0.0__b03f5f7f11d50a3a\System.Drawing.dll
// System.Windows.Forms --> C:\WINDOWS\Microsoft.Net\assembly\GAC_MSIL\System.Windows.Forms\v4.0_4.0.0.0__b77a5c561934e089\System.Windows.Forms.dll
//
// Release Notes:
// V1.6 (04/27/2024)
// * Fixed an error that occured when KeePass closed the NotesSearch Plugin after the timer for automatically 
// locking KeePass after X seconds of inactivity expired (Options --> Lock workspace after KeePass inactivity (seconds))
//
// V1.5 (04/20/2024)
// * Added change background of button "Save (Alt-Q)" to red if notes field was changed, ask for "Save Changes?" if notes field changed and you close the plugin 
//
// V1.4 (01/24/2024)
// * Added "Save (Alt-Q)" - Changes in the notes field can now be saved (with warning)
//
// V1.3 (10/13/2023)
// * URLs are now opened in the default browser when clicked
//
// V1.2 (09/25/2023)
// * Search-as-you-type added (start searching as soon as you type your search phrase)
//
// V1.1 (09/14/2023)
// * increase font size of notes field by 15% and the search-text-field by 50%
// * remove any whitespace from the search-text (right/left trim) 
//
// V1.0 (09/07/2023)
// * Initial release 
//
// Notes for version update:
// in AssemblyInfo.cs update: [assembly: AssemblyFileVersion("1.6")]
// on Homepage update file "https://besch-it.com/wp-content/uploads/2023/09/NotesSearchPluginVersion.txt"

using System;
using System.Drawing;
using System.Windows.Forms;
using KeePass.Plugins;
using KeePass.UI;
using KeePassLib;
using System.Diagnostics;
using System.Collections.Generic;

namespace NotesSearchPlugin
{
    public sealed class NotesSearchPluginExt : Plugin
    {
        public const string PluginVersion = "1.6 (04/27/24)";

        private IPluginHost m_host = null;
        private ToolStripMenuItem m_menuItem;
        private RichTextBox notesRichTextBox;
        private TextBox searchTextBox;
        private TextBox notFoundTextBox;
        private int lastSearchIndex = 0;
        private bool textChanged = false;
    	private bool ignoreTextChanged = true;
    
        public override bool Initialize(IPluginHost host)
        {
            if (host == null) return false;
            m_host = host;

            m_menuItem = new ToolStripMenuItem("NotesSearch");
            m_menuItem.Click += OnMenuSearchNotes;

            m_host.MainWindow.EntryContextMenu.Items.Add(m_menuItem);

            return true;
        }

        public override string UpdateUrl
        {
            get { return "https://besch-it.com/wp-content/uploads/2023/09/NotesSearchPluginVersion.txt"; }
        }

        private void OnMenuSearchNotes(object sender, EventArgs e)
        {
            PwEntry currentEntry = m_host.MainWindow.GetSelectedEntry(true);
            string notes = currentEntry.Strings.ReadSafe("Notes");

            if (string.IsNullOrWhiteSpace(notes))
            {
                MessageBox.Show("The notes field is empty!", "NotesSearch", MessageBoxButtons.OK, MessageBoxIcon.Information);
                return;
            }

            Form notesForm = new Form
            {
                Text = "NotesSearch V" + PluginVersion + " | www.besch-it.com | code@besch-it.com | Gerd Besch",
                Size = new Size(650, 850),
                FormBorderStyle = FormBorderStyle.Sizable,
                StartPosition = FormStartPosition.CenterParent
            };

            notesForm.Shown += (s, args) =>
            {
                searchTextBox.Focus();
                float currentSize = notesRichTextBox.Font.Size;
                float newSize = currentSize * 1.15f;
                notesRichTextBox.Font = new Font(notesRichTextBox.Font.FontFamily, newSize);
            };

            notesRichTextBox = new RichTextBox
            {
                Multiline = true,
                Dock = DockStyle.Fill,
                Text = notes
            };

            // Enable auto-link detection for URLs in the notes field
            notesRichTextBox.DetectUrls = true;
            notesRichTextBox.LinkClicked += NotesRichTextBox_LinkClicked;

            notesForm.Controls.Add(notesRichTextBox);

            notFoundTextBox = new TextBox
            {
                Dock = DockStyle.Top,
                Text = "No further occurrences of the search text were found!",
                ForeColor = Color.Red,
                TextAlign = HorizontalAlignment.Center,
                Visible = false
            };

            notesForm.Controls.Add(notFoundTextBox);
            
            searchTextBox = new TextBox
            {
                Dock = DockStyle.Top,
                Text = ""
            };

            float currentSizeSearchTextBox = searchTextBox.Font.Size;
            float newSizeSearchTextBox = currentSizeSearchTextBox * 1.5f;
            searchTextBox.Font = new Font(searchTextBox.Font.FontFamily, newSizeSearchTextBox);
            searchTextBox.Height = (int)(searchTextBox.Height * 1.5f);

            notesForm.Controls.Add(searchTextBox);

            searchTextBox.TextChanged += (s, args) =>
            {
                string searchText = searchTextBox.Text;
                ClearSelection();
                lastSearchIndex = 0;
                SearchText(searchText);
            };

            Button continueSearchButton = new Button
            {
                Dock = DockStyle.Top,
                Text = "Search next (Alt-&S)",
            };
            continueSearchButton.Click += (s, args) =>
            {
                if (!string.IsNullOrEmpty(searchTextBox.Text))
            		SearchText(searchTextBox.Text);
            };

            notesForm.Controls.Add(continueSearchButton);

            Button searchButton = new Button
            {
                Dock = DockStyle.Top,
                Text = "Search from top (Alt-&W)",
            };
            searchButton.Click += (s, args) =>
            {
            	if (!string.IsNullOrEmpty(searchTextBox.Text)) {
            		ClearSelection();
                	lastSearchIndex = 0;
                	SearchText(searchTextBox.Text);
            	}
            };

            notesForm.Controls.Add(searchButton);

            searchTextBox.KeyPress += (s, args) =>
            {
                if (args.KeyChar == (char)Keys.Return)
                {
                    ClearSelection();
                    lastSearchIndex = 0;
                    SearchText(searchTextBox.Text);
                    args.Handled = true;
                }
            };
            
            
            Button saveButton = new Button
            {
                Dock = DockStyle.Top,
                Text = "Save (Alt-&Q)",
            };
            
            saveButton.Click += (s, args) =>
            {
                if (IsNoteTextChanged())
                {
                    DialogResult result = MessageBox.Show("Do you want to save your changes?", "Save Changes", MessageBoxButtons.YesNo, MessageBoxIcon.Question);

                    if (result == DialogResult.Yes)
                    {
                        SaveNoteChanges(notes);
                        textChanged = false; // Setze den Textänderungsstatus zurück, nachdem die Änderungen gespeichert wurden
		                saveButton.BackColor = SystemColors.ControlLight; // Ändere die Hintergrundfarbe des Save-Buttons sofort
                    }
                }
            };
            
            notesForm.Controls.Add(saveButton);
            
            notesRichTextBox.TextChanged += (s, args) =>
	        {
            	if (!ignoreTextChanged)
	            {

            		if (IsNoteTextChanged()) {
	                	saveButton.BackColor = Color.Red; // Ändere die Hintergrundfarbe des Save-Buttons sofort
            		}
                	else
                	{
	                	saveButton.BackColor = SystemColors.ControlLight; // Ändere die Hintergrundfarbe des Save-Buttons sofort
            		}
            	}
	            else
	            {
	                ignoreTextChanged = false;
	            }
	        };
            
            notesForm.FormClosing += (s, args) =>
        	{
            	ignoreTextChanged = true; // Verhindere, dass das TextChanged-Ereignis beim Schließen des Formulars ausgelöst wird

           		if (IsNoteTextChanged()) {
            		DialogResult result = MessageBox.Show("Do you want to save your changes?", "Save Changes", MessageBoxButtons.YesNoCancel, MessageBoxIcon.Question);
            		
            		if (result == DialogResult.Yes)
                    {
                        SaveNoteChanges(notes);
                        textChanged = false; // Setze den Textänderungsstatus zurück, nachdem die Änderungen gespeichert wurden
		                saveButton.BackColor = SystemColors.ControlLight; // Ändere die Hintergrundfarbe des Save-Buttons sofort
                    }
            		else if (result == DialogResult.Cancel)
                	{
                    	args.Cancel = true; // Abbrechen des Schließvorgangs
                	}
            	}
	        };   

            notesForm.ShowDialog();
        }
        
        private bool IsNoteTextChanged()
		{
    		PwEntry currentEntry = m_host.MainWindow.GetSelectedEntry(true);

    		// Prüfen, ob currentEntry null ist, bevor darauf zugegriffen wird
    		if (currentEntry != null)
    		{
				string originalNotes = currentEntry.Strings.ReadSafe("Notes");
        		bool changed = originalNotes != notesRichTextBox.Text;
        		return changed;
    		}
    		else
    		{
        		// Wenn currentEntry null ist, geben Sie false zurück oder führen Sie andere geeignete Logik aus
        		return false;
    		}
		}
   
        private void SaveNoteChanges(string notes)
        {
			PwDatabase database = m_host.Database;

		    if (database != null)
		    {
		        PwEntry currentEntry = m_host.MainWindow.GetSelectedEntry(true);

		        if (currentEntry != null)
		        {
		            // Hier wird der Eintrag aktualisiert
		            string updatedNotes = notesRichTextBox.Text;
		            KeePassLib.Security.ProtectedString newNotes = new KeePassLib.Security.ProtectedString(true, updatedNotes);
		            currentEntry.Strings.Set("Notes", newNotes);
		
		            // Aktualisieren des Änderungsdatums
		            currentEntry.Touch(true);
		
		            // Speichern der Änderungen in der Datenbank
		            database.Save(null); // Hier speichert die Datenbank alle Änderungen
		            
		            MessageBox.Show("Changes successfully saved!", "Save Changes",
		                MessageBoxButtons.OK, MessageBoxIcon.Information);
		        }
		        else
		        {
		            MessageBox.Show("No entry selected", "Save",
		                MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
		        }
		    }
        }

        private void NotesRichTextBox_LinkClicked(object sender, LinkClickedEventArgs e)
        {
            // Open the clicked URL in the default web browser
            try
            {
                Process.Start(e.LinkText);
            }
            catch (Exception ex)
            {
                MessageBox.Show("Error opening URL: " + ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        public override void Terminate()
        {
            notesRichTextBox.Dispose();
            searchTextBox.Dispose();
            notFoundTextBox.Dispose();
        }

        private void SearchText(string searchText)
        {
            string notesText = notesRichTextBox.Text;

            if (!string.IsNullOrEmpty(searchText))
            {
                searchText = searchText.Trim();
                int startIndex = notesText.IndexOf(searchText, lastSearchIndex, StringComparison.OrdinalIgnoreCase);

                if (startIndex >= 0)
                {
		            ClearSelection();

                	notesRichTextBox.Select(startIndex, searchText.Length);
                    notesRichTextBox.SelectionColor = Color.Red;

                    float currentSize = notesRichTextBox.SelectionFont.Size;
                    float newSize = currentSize * 1.4f;
                    notesRichTextBox.SelectionFont = new Font(notesRichTextBox.SelectionFont.FontFamily, newSize);

                    notesRichTextBox.ScrollToCaret();
                    lastSearchIndex = startIndex + searchText.Length;
                    notFoundTextBox.Visible = false;
                }
                else
                {
                    notesRichTextBox.ScrollToCaret();
                	notFoundTextBox.Visible = true;
                }
            }
        }

        private void ClearSelection()
        {
            notesRichTextBox.SelectionStart = 0;
            notesRichTextBox.SelectionLength = notesRichTextBox.Text.Length;
            notesRichTextBox.SelectionColor = notesRichTextBox.ForeColor;
            notesRichTextBox.SelectionFont = notesRichTextBox.Font;
        }
    }
}
